function r=SpikeSlab_givenZ(y,x,u,abeta,bbeta,Abeta,Bbeta,M,N,zstar,gam2in);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function runs the baseline spike-andslab model of JPT (2018),
% conditional on a specific z
% y: response variable
% x: all predictors
% abeta and bbeta: parameters of beta-prior on q (baseline choice: 1 and 1)
% Abeta and Bbeta: parameters of beta-prior on R2 (baseline choice: 1 and 1)
% M and N: total number of draws, and initial draws to be discarded
% zstar: specif model in the conditioning set
% gam2in: value of gam2 to be used to initialize the Gibbs sampler
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty(u); l=0; else; l=size(u,2); end;
[T,k]=size(x);
varx=var(x(:,1),1);

% other choices
% edgesq=[0:.001:.1 .11:.01:.9 .901:.001:1];
% edgesR2=[0:.001:.1 .11:.01:.9 .901:.001:1];

edgesq=[0:.01:1];
edgesR2=[0:.01:1];

% storage and preliminary calculations
if l>0; store_phi=zeros(l,M); end
store_B=zeros(k,M);
store_s2=zeros(M,1);
store_gam=zeros(M,1);
store_z=zeros(k,M);
store_q=zeros(M,1);
store_R2=zeros(M,1);
areaq=edgesq(2:end)-edgesq(1:end-1);
areaR2=edgesR2(2:end)-edgesR2(1:end-1);
areaqR2=repmat(areaq,length(areaR2),1).*repmat(areaR2',1,length(areaq)); % areaq.*areaR2';
intq=edgesq(1:end-1)+areaq/2;
intR2=edgesR2(1:end-1)+areaR2/2;
INTQ=repmat(intq,length(intR2),1);
INTR2=repmat(intR2',1,length(intq));

xx=x'*x; xy=x'*y; yx=y'*x; yy=y'*y;
if l>0; xu=x'*u; yu=y'*u; uu=u'*u; invuu=u'*u\eye(l); cholinvuu=chol(invuu); ux=u'*x; invuuuy=invuu*u'*y; invuuux=invuu*u'*x;
elseif l==0; xu=zeros(k,1); yu=zeros(1,1); uu=zeros(1,1); ux=zeros(1,k); phi=0; end
    
QR2=INTQ.*(1-INTR2)./INTR2;
for ttt=0:k
    prodINT(:,:,ttt+1)=INTQ.^(ttt+ttt/2+abeta-1).*(1-INTQ).^(k-ttt+bbeta-1).*...
        INTR2.^(Abeta-1-ttt/2).*(1-INTR2).^(ttt/2+Bbeta-1).*areaqR2;
end

% starting values (Ridge using z)
if l>0; phi0=invuuuy; resy=y-u*phi0; store_phi(:,1)=phi0; else; resy=y; end
omegainv=eye(sum(1-zstar))/gam2in;
b0=(x(:,logical(1-zstar))'*x(:,logical(1-zstar))+omegainv)\(x(:,logical(1-zstar))'*y);
store_B(logical(1-zstar),1)=b0; b=zeros(k,1); b(logical(1-zstar))=b0;
s2=sum((resy-x*b).^2)/T; store_s2(1)=s2;
z=(b==0); store_z(:,1)=z;
tau=sum(1-z);



% Gibbs sampling
for i=2:M
    if i==1000*floor(.001*i);
        i
    end
    
    % draw q and R2
    %     pr_qR2=exp(-(.5*k*varx*b'*b/s2).*INTQ.*(1-INTR2)./INTR2).*...
    %         [INTQ.^(tau+tau/2+abeta-1).*(1-INTQ).^(k-tau+bbeta-1)].*...
    %         [INTR2.^(Abeta-1-tau/2).*(1-INTR2).^(tau/2+Bbeta-1)];
    pr_qR2=exp(-(.5*k*varx*b'*b/s2).*QR2).*squeeze(prodINT(:,:,tau+1));
    
    %aux=randsample(length(intq)*length(intR2),1,true,pr_qR2(:));
    cdf_qR2=cumsum(pr_qR2(:)/sum(pr_qR2(:)));
    aux=sum(cdf_qR2<rand(1))+1;
    
    q=INTQ(aux);
    store_q(i)=q;
    R2=INTR2(aux);
    store_R2(i)=R2;
    gam=sqrt((1/(k*varx*q))*R2/(1-R2));
    store_gam(i)=gam;
    
    % draw phi
    if l>0;
        phihat=invuuuy-invuuux*store_B(:,i-1);
        Vphi=invuu*s2;
        phi=(randn(1,l)*sqrt(s2)*cholinvuu)'+phihat;
        %phi=mvnrnd(phihat,Vphi)';
        store_phi(:,i)=phi';
        % ytil=y-u*phi';
    end
    
    % Gibbs
%     for j=1:k
%         z0=z; z0(j)=0;
%         z1=z; z1(j)=1;
%         tau0=sum(1-z0);
%         tau1=sum(1-z1);
%         % W0=x(:,logical(1-z0))'*x(:,logical(1-z0))+eye(tau0)/gam^2;        
%         W0=xx(logical(1-z0),logical(1-z0))+eye(tau0)/gam^2;
%         % W1=x(:,logical(1-z1))'*x(:,logical(1-z1))+eye(tau1)/gam^2;        
%         W1=xx(logical(1-z1),logical(1-z1))+eye(tau1)/gam^2;
%         % bhat0=W0\(x(:,logical(1-z0))'*ytil);        
%         bhat0=W0\(xy(logical(1-z0))-xu(logical(1-z0),:)*phi);
%         % bhat1=W1\(x(:,logical(1-z1))'*ytil);
%         bhat1=W1\(xy(logical(1-z1))-xu(logical(1-z1),:)*phi);
%         
%         % note: 2*sum(log(diag(chol(W0)))) improves speed an accuracy of log(det(W0))
%         % log_pr_z0=tau0*log(q)+(k-tau0)*log(1-q)-tau0*log(gam)-.5*2*sum(log(diag(chol(W0))))-.5*T*log(ytil'*(ytil-x(:,logical(1-z0))*bhat0));        
%         log_pr_z0=tau0*log(q)+(k-tau0)*log(1-q)-tau0*log(gam)-.5*2*sum(log(diag(chol(W0))))...
%             -.5*T*log(yy-2*yu*phi+phi'*uu*phi-yx(logical(1-z0))*bhat0+phi'*ux(:,logical(1-z0))*bhat0);
%         %log_pr_z1=tau1*log(q)+(k-tau1)*log(1-q)-tau1*log(gam)-.5*2*sum(log(diag(chol(W1))))-.5*T*log(ytil'*(ytil-x(:,logical(1-z1))*bhat1));        
%         log_pr_z1=tau1*log(q)+(k-tau1)*log(1-q)-tau1*log(gam)-.5*2*sum(log(diag(chol(W1))))...
%             -.5*T*log(yy-2*yu*phi+phi'*uu*phi-yx(logical(1-z1))*bhat1+phi'*ux(:,logical(1-z1))*bhat1);
%         
%         z(j)=0+(rand(1)>(1/(exp(log_pr_z1-log_pr_z0)+1)));
%     end
    z=zstar;

    tau=sum(1-z);
    % W=x(:,logical(1-z))'*x(:,logical(1-z))+eye(tau)/gam^2;    
    W=xx(logical(1-z),logical(1-z))+eye(tau)/gam^2;
    % bhat=W\(x(:,logical(1-z))'*ytil);
    bhat=W\(xy(logical(1-z))-xu(logical(1-z),:)*phi);
    
    store_z(:,i)=z;
    
    % draw s2
    % niu = ytil'*ytil-bhat'*W*bhat;
    niu = yy-2*yu*phi+phi'*uu*phi-bhat'*W*bhat;
    s2=1./gamrnd(T/2,2/niu);
    store_s2(i)=s2;
    
    % draw b
    b=(randn(1,tau)*chol(s2*eye(tau)/W))'+bhat;
    store_B(logical(1-z),i)=b;
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% processing the results
if l>0; r.store_phi=store_phi; end
r.store_B=store_B;
r.store_s2=store_s2;
r.store_gam=store_gam;
r.store_z=store_z;
r.store_q=store_q;
r.store_R2=store_R2;