%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code used to produce the Log Predictive Score and the Mean Squared error 
% in the out-of-sample exercise
% (we recommend to run this code only with access to parallel computing)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear all
rng(10)

load ../../../data/Finance1/Goyal.mat
load ../../../data/Finance1/CvSamplesFinance1.mat
data=[x y]; clear x X y Y;
data=[ones(size(data,1),1) zscore(data,1)];
l=1;
lns=1;
Mcv = size(EstSmplCv,2);

if l>0; U=data(:,1:l); else; U=[]; end
X=data(:,l+1:end-1);
[T k]=size(X);
Y=data(:,end);
clear data

smallfraction=.0005;

abeta=1;
bbeta=1;
Abeta=1;
Bbeta=1;

M0=1100000;
N0=100000;
M1=M0;
N1=N0;
M2=M0;
N2=N0;

MAXS=[5 10 k 1000];     % 1000 corresponds to the dense model

Mscore0=zeros(Mcv,length(MAXS)); SQdev0=zeros(Mcv,length(MAXS));
Mscore1=zeros(Mcv,length(MAXS)); SQdev1=zeros(Mcv,length(MAXS));
Mscore2=zeros(Mcv,length(MAXS)); SQdev2=zeros(Mcv,length(MAXS));
Mscore3=zeros(Mcv,length(MAXS)); SQdev3=zeros(Mcv,length(MAXS));
ZSTAR=zeros(Mcv,k,length(MAXS));
GAM2STAR=zeros(Mcv,length(MAXS));
NEWMAXS=zeros(Mcv,length(MAXS));

parpool(20)

tic
parfor i=1:Mcv
    %% run the baseline spike-and-slab model
    SmplIn  = EstSmplCv{i}; 
    SmplOut = EvalSmplCv{i};
    Tin=length(SmplIn);
    Tout=length(SmplOut);
    
    if l>0; [ut,Mu,Su] = zscore(U(SmplIn,lns+1:end),1); else; ut=[]; end
    if lns>0; ut=[U(SmplIn,1:lns) ut]; Mu=[zeros(1,lns) Mu]; Su=[ones(1,lns) Su]; end 
    [x,Mx,Sx] = zscore(X(SmplIn,:),1);
    [y,My,Sy] = zscore(Y(SmplIn,:),1);
    
    r0=SpikeSlabGLP_forSimulationsForecasts(y,x,ut,abeta,bbeta,Abeta,Bbeta,M0,N0);
    
    sqdev0=zeros(1,length(MAXS)); mscore0=zeros(1,length(MAXS));
    sqdev1=zeros(1,length(MAXS)); mscore1=zeros(1,length(MAXS));
    sqdev2=zeros(1,length(MAXS)); mscore2=zeros(1,length(MAXS));
    sqdev3=zeros(1,length(MAXS)); mscore3=zeros(1,length(MAXS));
    Zstar=zeros(1,k,length(MAXS));
    GAM2star=zeros(1,length(MAXS));
    NEWmaxS=zeros(1,length(MAXS));
    
    countmaxS=0;
    for maxS=MAXS;
        
        countmaxS=countmaxS+1;
        
        if maxS<=k
            % compute predictive score and point forecast
            indmaxS=find(sum(1-r0.store_z(:,N0+1:M0))<=maxS);
            newmaxS=maxS;
            % if 0 models with maxS predictors or less, use the smallest .1%
            if length(indmaxS)<smallfraction*(M0-N0)
                while length(indmaxS)<smallfraction*(M0-N0)
                    newmaxS=newmaxS+1;
                    indmaxS=find(sum(1-r0.store_z(:,N0+1:M0))<=newmaxS);
                end
            end
            NEWmaxS(countmaxS)=newmaxS;
            
            Mfct = My + ((X(SmplOut,:)-Mx)./Sx*r0.store_B(:,N0+indmaxS))*Sy;
            if l>0; Mfct=Mfct+((U(SmplOut,:)-Mu)./Su*r0.store_phi(:,N0+indmaxS))*Sy; end
            Vfct=r0.store_s2(N0+indmaxS)'*Sy^2;
            sqdev0(countmaxS)=mean((Y(SmplOut)-mean(Mfct,2)).^2);
            % mscore0(countmaxS)=mean(log(mean(normpdf(Y(SmplOut),Mfct,repmat(sqrt(Vfct),Tout,1)),2))); % baseline version
            % logaux=logofnormpdf(Y(SmplOut),Mfct,repmat(sqrt(Vfct),Tout,1)); % more robust version for super small values of the density
            % mscore0(countmaxS)=mean(mean(logaux,2)+log(mean(exp(logaux-mean(logaux,2)),2))); 
            mscoreaux=zeros(Tout,1).*NaN; % more robust version for larger Mfct
            for jout=1:Tout
                logaux=logofnormpdf(Y(SmplOut(jout)),Mfct(jout,:),sqrt(Vfct));
                mscoreaux(jout)=mean(logaux)+log(mean(exp(logaux-mean(logaux))));
            end 
            mscore0(countmaxS)=mean(mscoreaux);
  
            [MaxMl,PosMax] = max(real(r0.PYZ(N0+indmaxS)));
            zstar=r0.store_z(:,N0+indmaxS(PosMax))';
            Zstar(1,:,countmaxS)=zstar;
            
            % compute median gam2 for initialization of next step
            gam2aux=r0.store_gam(N0+indmaxS).^2;
            Z_maxS=r0.store_z(:,N0+indmaxS);
            gam2in=median(gam2aux(find(sum(Z_maxS~=repmat(zstar',1,size(Z_maxS,2)))==0)));
        
        else
            % dense model case
            zstar=zeros(1,k);
            Zstar(1,:,countmaxS)=zstar;
            gamaux=r0.store_gam(N0+find(sum(1-r0.store_z(:,N0+1:end))==k));
            newk=k;
            while length(gamaux)<smallfraction*(M0-N0)
                newk=newk-1;
                gamaux=r0.store_gam(N0+find(sum(1-r0.store_z(:,N0+1:end))>=newk));
            end
            gam2in=median(gamaux).^2;
        end
        
        % avoid repeating estimation when selected model is the same
        if countmaxS>=2 & sum(abs(squeeze(Zstar(1,:,countmaxS-1))-zstar))==0;
            sqdev1(countmaxS)=sqdev1(countmaxS-1); mscore1(countmaxS)=mscore1(countmaxS-1);
            GAM2star(countmaxS)=GAM2star(countmaxS-1);
            sqdev2(countmaxS)=sqdev2(countmaxS-1); mscore2(countmaxS)=mscore2(countmaxS-1);
            sqdev3(countmaxS)=sqdev3(countmaxS-1); mscore3(countmaxS)=mscore3(countmaxS-1);
        else
            %% run the spike-and-slab model conditional on the modal z
            r1=SpikeSlab_givenZ(y,x,ut,abeta,bbeta,Abeta,Bbeta,M1,N1,zstar,gam2in);
            
            % compute predictive score and point forecast
            Mfct = My + ((X(SmplOut,:)-Mx)./Sx*r1.store_B(:,N1+1:end))*Sy;
            if l>0; Mfct=Mfct+((U(SmplOut,:)-Mu)./Su*r1.store_phi(:,N1+1:end))*Sy; end
            Vfct=r1.store_s2(N1+1:end)'*Sy^2;
            sqdev1(countmaxS)=mean((Y(SmplOut)-mean(Mfct,2)).^2);
            % mscore1(countmaxS)=mean(log(mean(normpdf(Y(SmplOut),Mfct,repmat(sqrt(Vfct),Tout,1)),2)));
            mscoreaux=zeros(Tout,1).*NaN;
            for jout=1:Tout
                logaux=logofnormpdf(Y(SmplOut(jout)),Mfct(jout,:),sqrt(Vfct));
                mscoreaux(jout)=mean(logaux)+log(mean(exp(logaux-mean(logaux))));
            end 
            mscore1(countmaxS)=mean(mscoreaux);
            
            % computation of the modal q, R2, and corresponding gam2
            [dens XI]=ksdensity([r1.store_q(N1+1:end) r1.store_R2(N1+1:end)],'Support',[0 0;1 1]);
            [junk,indpos]=max(dens);
            qstar=XI(indpos,1);
            R2star=XI(indpos,2);
            gam2star=((1/(k*var(x(:,1),1)*qstar))*R2star/(1-R2star));
            GAM2star(countmaxS)=gam2star;
            
            %% run the spike-and-slab model conditional on the modal z and gam2
            r2=SpikeSlab_givenZqR2(y,x,ut,M2,N2,zstar,gam2star);
            
            % compute predictive score and point forecast
            Mfct = My + ((X(SmplOut,:)-Mx)./Sx*r2.store_B(:,N2+1:end))*Sy;
            if l>0; Mfct=Mfct+((U(SmplOut,:)-Mu)./Su*r2.store_phi(:,N2+1:end))*Sy; end
            Vfct= r2.store_s2(N2+1:end)'*Sy^2;
            sqdev2(countmaxS)=mean((Y(SmplOut)-mean(Mfct,2)).^2);
            % mscore2(countmaxS)=mean(log(mean(normpdf(Y(SmplOut),Mfct,repmat(sqrt(Vfct),Tout,1)),2)));
            mscoreaux=zeros(Tout,1).*NaN;
            for jout=1:Tout
                logaux=logofnormpdf(Y(SmplOut(jout)),Mfct(jout,:),sqrt(Vfct));
                mscoreaux(jout)=mean(logaux)+log(mean(exp(logaux-mean(logaux))));
            end 
            mscore2(countmaxS)=mean(mscoreaux);
            
            %% run the spike-and-slab model conditional on the modal z, gam2, beta and sigma2
            omegainv=eye(sum(1-zstar)+l)/gam2star; if l>0; omegainv(1:l,1:l)=0*eye(l); end
            bhat=([ut x(:,logical(1-zstar))]'*[ut x(:,logical(1-zstar))]+omegainv)\([ut x(:,logical(1-zstar))]'*y);
            Bhat=zeros(k,1); Bhat(logical(1-zstar))=bhat(l+1:end); phihat=bhat(1:l);
            epshat=y-[ut x(:,logical(1-zstar))]*bhat;
            s2hat=(epshat'*epshat+bhat'*omegainv*bhat)/(length(epshat)-l+2);
            Mfct = My + (X(SmplOut,:)-Mx)./Sx*Bhat*Sy;
            if l>0; Mfct=Mfct+(U(SmplOut,:)-Mu)./Su*phihat*Sy; end
            Vfct= s2hat*Sy^2;
            sqdev3(countmaxS)=mean((Y(SmplOut)-Mfct).^2);
            % mscore3(countmaxS)=mean(log(normpdf(Y(SmplOut),Mfct,sqrt(Vfct))));
            mscoreaux=zeros(Tout,1).*NaN;
            for jout=1:Tout
                logaux=logofnormpdf(Y(SmplOut(jout)),Mfct(jout,:),sqrt(Vfct));
                mscoreaux(jout)=mean(logaux)+log(mean(exp(logaux-mean(logaux))));
            end 
            mscore3(countmaxS)=mean(mscoreaux);
        end
        
    end
    
    SQdev0(i,:)=sqdev0; Mscore0(i,:)=mscore0;
    SQdev1(i,:)=sqdev1; Mscore1(i,:)=mscore1;
    SQdev2(i,:)=sqdev2; Mscore2(i,:)=mscore2;
    SQdev3(i,:)=sqdev3; Mscore3(i,:)=mscore3;
    ZSTAR(i,:,:)=Zstar;
    GAM2STAR(i,:)=GAM2star;
    NEWMAXS(i,:)=NEWmaxS;
    
end
timeCV=toc

clearvars -except SQdev0 SQdev1 SQdev2 SQdev3 Mscore0 Mscore1 Mscore2 Mscore3 NEWMAXS

% save ../BMAandSSResults/NEWforecasts/Finance1_51CV_1000K.mat;