%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code used to produce the Log Predictive Score and the Mean Squared error 
% in the Lasso out-of-sample exercise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
rng(10)

load ../../../data/Micro2/Data1stStageEminentDomain_u80.mat
load ../../../data/Micro2/CvSamplesMicro2.mat
dataset='EminentDomain';
data=Data1stStageEminentDomain_u80; clear Data1stStageEminentDomain_u80;
pU=data(:,1:80);
data(:,1:80)=[];
l=0;
maxS=138;
Mcv = 5*20;

if l>0; origU=data(:,1:l); else; origU=[]; end
origX=data(:,l+1:end-1);
[T k]=size(origX);
origY=data(:,end);
clear data

Nmodels=8;
ModelNames=['fix5','   fix10','   asymptotic','   asymptoticSR','   CV2','   CV5','   CV10','    PCregression'];
ModelsToRun=[1 2 3 5 6 7];

Mscore2=zeros(Mcv,Nmodels); SQdev2=zeros(Mcv,Nmodels);
Mscore3=zeros(Mcv,Nmodels);
ZSTAR=zeros(Mcv,k,Nmodels);

tic
for i=1:Mcv
    i
    %% selection of the CV sample
    SmplIn  = EstSmplCv{i};
    SmplOut = EvalSmplCv{i};
    Tin=length(SmplIn);
    Tout=length(SmplOut);
    
    % preliminary ols regression on larger sets of fixed controls (for Micro1 and Micro2, as in BCH and BCCH original papers)
    if strcmp(dataset,'abortion')|strcmp(dataset,'EminentDomain')
        pr=ols1(origY(SmplIn),[pU(SmplIn,:)]);
        Y=origY-pU*pr.bhatols;
        prx=olsblock(origX(SmplIn,:),[pU(SmplIn,:)]);
        X=origX-pU*prx.bhatols;
        if l>0
            pru=ols1(origU(SmplIn,:),[pU(SmplIn,:)]);
            U=origU-pU*pru.bhatols;
        else
            U=[];
        end
    else
        Y=origY; X=origX; U=origU;
    end
    UX=[U X];
    U1000X=[U*1000 X];
    
    
    %% standardization of the variables in the training sample
    if l>0; [ut,Mu,Su] = zscore(U(SmplIn,:),1); SmplInU=SmplIn; else; ut=[]; Mu=[]; Su=[]; SmplInU=[]; end
    [x,Mx,Sx] = zscore(X(SmplIn,:),1);
    [y,My,Sy] = zscore(Y(SmplIn,:),1);
    ux=[ut x];
    Mux=[Mu Mx];
    Sux=[Su Sx];
    
    
    %% post lasso with 5 predictors
    fixS=5;
    [bfix]=Lars([ut*1000 x],y);
    is=find(bfix(1+fixS+l,:)~=0);
    ZSTAR(i,is(l+1:end),1)=1;
    
    if l==0;
        Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
        yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
    elseif l>0;
        Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
        yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
    end
    %SQdev2(i,1)=mean((Y(SmplOut)-Mfct).^2);
    
    % log predictive scores
    ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
    df=Tin-2;
    scale=sqrt(ssr/df);
    %scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
    %scale=sqrt((scale+1)*ssr/df);
    logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
    Mscore2(i,1)=mean(logtpdf);
    Mscore3(i,1)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
    
    uniqueY=unique(origY); 
    temp=zeros(Tout,length(uniqueY));
    for ii=1:length(uniqueY)
        temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
    end
    [junk maxscore]=max(temp');
    SQdev2(i,1)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
   
    
    %% post lasso with 10 predictors
    fixS=10;
    is=find(bfix(1+fixS+l,:)~=0);
    ZSTAR(i,is(l+1:end),2)=1;
    
    if l==0;
        Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
        yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
    elseif l>0;
        Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
        yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+fixS+l,:)'+My;
    end
    %SQdev2(i,2)=mean((Y(SmplOut)-Mfct).^2);
    
    % log predictive scores
    ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
    df=Tin-2;
    scale=sqrt(ssr/df);
    logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
    Mscore2(i,2)=mean(logtpdf);
    Mscore3(i,2)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
    
    uniqueY=unique(origY); 
    temp=zeros(Tout,length(uniqueY));
    for ii=1:length(uniqueY)
        temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
    end
    [junk maxscore]=max(temp');
    SQdev2(i,2)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
    
    
    %% post lasso with the asymptotic x-dependent selection of BCH (alg 2)
    if sum(ModelsToRun==3)>0
        M=5000;
        aux=NaN(M,1);
        for rep=1:M
            aux(rep)=Tin*max(abs(mean(x.*repmat(normrnd(0,1,Tin,1),1,k))));
        end
        % quantile=prctile(aux,100*(1-.1/log(max([T,k])))); % recommendation in BCCH Econometrica
        quantile=prctile(aux,95); % recommendation in BCH World congress
        rit=IteratedLassoBCH(y,x,ut,quantile,'algorithm1');
        is=find(rit.blassoBCHxdep~=0);

        if l==0;
            Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*rit.blassoBCHxdep+My;
            yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*rit.blassoBCHxdep+My;
        elseif l>0;
            Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*rit.blassoBCHxdep+My;
            yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*rit.blassoBCHxdep+My;
        end

%         quantile=sqrt(Tin)*norminv(1-.05/(2*k));
%         rit=IteratedLassoBCH(y,x,ut,quantile,'algorithm1');
%         is=find(rit.blassoBCHindep~=0);
%                 
%         if l==0;
%             Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*rit.blassoBCHindep+My;
%             yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*rit.blassoBCHindep+My;
%         elseif l>0;
%             Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*rit.blassoBCHindep+My;
%             yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*rit.blassoBCHindep+My;
%         end
        
        ZSTAR(i,is(l+1:end),3)=1;
        %SQdev2(i,3)=mean((Y(SmplOut)-Mfct).^2);
    
        % log predictive scores
        ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
        df=Tin-2;
        scale=sqrt(ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,3)=mean(logtpdf);
        Mscore3(i,3)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,3)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
        
    end
    
    
    %% Square-root post lasso with the asymptotic x-dependent selection of BCH
    if sum(ModelsToRun==4)>0
        ALPHA = 0.05;
        c = 1.1;
        M=5000;
        aux=NaN(M,1);
        for rep=1:M
            gs = normrnd(0,1,Tin,1);
            aux(rep)=Tin*max(abs(mean(x.*repmat(gs,1,k))))/sqrt(mean(gs.^2));
        end
        quantile=prctile(aux,100*(1-ALPHA)); % recommendation in BCH World congress
        
        lambda = c*quantile;
        % lambda = c*sqrt(Tin)*norminv(1 - ( ALPHA/(2*k) ), 0,1);
        [betaSQ, sSQ] = SqrtLassoIterative([ut*1000 x], y, lambda, ones(k+l,1));
        
        is=find(betaSQ~=0);
        %is=find(rit.blassoBCHindep~=0);
        ZSTAR(i,is(l+1:end),4)=1;
        
        if l==0;
            Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*betaSQ+My;
            yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*betaSQ+My;
        elseif l>0;
            Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*betaSQ+My;
            yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*betaSQ+My;
        end
        %SQdev2(i,4)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
        df=Tin-2;
        scale=sqrt(ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,4)=mean(logtpdf);
        Mscore3(i,4)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,4)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
        
    end
    
    
    %% post lasso with 2-fold CV
    if sum(ModelsToRun==5)>0
        Kfold=2;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=LassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),5)=1;
        
        if l==0;
            Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        elseif l>0;
            Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        end
        %SQdev2(i,5)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
        df=Tin-2;
        scale=sqrt(ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,5)=mean(logtpdf);
        Mscore3(i,5)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,5)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
        
    end
    
    
    %% post lasso with 5-fold CV
    if sum(ModelsToRun==6)>0
        Kfold=5;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=LassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),6)=1;
        
        if l==0;
            Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        elseif l>0;
            Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        end
        %SQdev2(i,6)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
        df=Tin-2;
        scale=sqrt(ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,6)=mean(logtpdf);
        Mscore3(i,6)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,6)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);

    end
    
    
    %% post lasso with 10-fold CV
    if sum(ModelsToRun==7)>0
        Kfold=10;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=LassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),7)=1;
        
        if l==0;
            Mfct=[(X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[(X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        elseif l>0;
            Mfct=[1000*(U(SmplOut,:)-Mu)./Su (X(SmplOut,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
            yhat=[1000*(U(SmplIn,:)-Mu)./Su (X(SmplIn,:)-Mx)./Sx]*Sy*bfix(1+rCV.s+l,:)'+My;
        end
        %SQdev2(i,7)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=(Y(SmplIn)-yhat)'*(Y(SmplIn)-yhat);
        df=Tin-2;
        scale=sqrt(ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,7)=mean(logtpdf);
        Mscore3(i,7)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/Tin)));
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,7)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);        
        
    end
    
end
timeCVlasso=toc

clearvars -except SQdev0 SQdev1 SQdev2 SQdev3 Mscore0 Mscore1 Mscore2 Mscore3 NEWMAXS ModelNames

% save ../LassoResults/NEWforecasts/LassoFcstGaussMicro2.mat;