%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code used to produce the Log Predictive Score and the Mean Squared error 
% in the Post Lasso out-of-sample exercise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
rng(10)

load ../../../data/Finance1/Goyal.mat
load ../../../data/Finance1/CvSamplesFinance1.mat
data=[x y]; clear x X y Y;
data=[zscore(data,1)];
l=0;
maxS=16;
Mcv = size(EstSmplCv,2);

if l>0; origU=data(:,1:l); else; origU=[]; end
origX=data(:,l+1:end-1);
[T k]=size(origX);
origY=data(:,end);
clear data

Nmodels=8;
ModelNames=['fix5','   fix10','   asymptotic','   asymptoticSR','   CV2','   CV5','   CV10','    PCregression'];
ModelsToRun=[1 2 3 4 5 6 7 8];

Mscore2=zeros(Mcv,Nmodels); SQdev2=zeros(Mcv,Nmodels);
Mscore3=zeros(Mcv,Nmodels);
ZSTAR=zeros(Mcv,k,Nmodels);

tic
for i=1:Mcv
    i
    %% selection of the CV sample
    SmplIn  = EstSmplCv{i};
    SmplOut = EvalSmplCv{i};
    Tin=length(SmplIn);
    Tout=length(SmplOut);
    
    % preliminary ols regression on larger sets of fixed controls (for Micro1 and Micro2, as in BCH and BCCH original papers)
    if strcmp(dataset,'abortion')|strcmp(dataset,'EminentDomain')
        pr=ols1(origY(SmplIn),[pU(SmplIn,:)]);
        Y=origY-pU*pr.bhatols;
        pr=olsblock(origX(SmplIn,:),[pU(SmplIn,:)]);
        X=origX-pU*pr.bhatols;
        if l>0
            pr=ols1(origU(SmplIn,:),[pU(SmplIn,:)]);
            U=origU-pU*pr.bhatols;
        else
            U=[];
        end
    else
        Y=origY; X=origX; U=origU; 
    end
    UX=[U X];

    
    %% standardization of the variables in the training sample
    if l>0; [ut,Mu,Su] = zscore(U(SmplIn,:),1); SmplInU=SmplIn; else; ut=[]; Mu=[]; Su=[]; SmplInU=[]; end
    [x,Mx,Sx] = zscore(X(SmplIn,:),1);
    [y,My,Sy] = zscore(Y(SmplIn,:),1);
    ux=[ut x];
    Mux=[Mu Mx];
    Sux=[Su Sx];
    
    
    %% post lasso with 5 predictors
    fixS=5;
    [bfix]=Lars([ut*1000 x],y);
    is=find(bfix(1+fixS+l,:)~=0);
    ZSTAR(i,is(l+1:end),1)=1;
    
    % squared deviations
    r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
    Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
    SQdev2(i,1)=mean((Y(SmplOut)-Mfct).^2);
    
    % log predictive scores
    ssr=r.resols'*r.resols;
    df=Tin-length(r.bhatols)-2;
    scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
    scale=sqrt((scale+1)*ssr/df);
    logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
    Mscore2(i,1)=mean(logtpdf);
    Mscore3(i,1)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));    
    
    
    %% post lasso with 10 predictors
    fixS=10;
    is=find(bfix(1+fixS+l,:)~=0);
    ZSTAR(i,is(l+1:end),2)=1;
    
    % squared deviations
    r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
    Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
    SQdev2(i,2)=mean((Y(SmplOut)-Mfct).^2);
    
    % log predictive scores
    ssr=r.resols'*r.resols;
    df=Tin-length(r.bhatols)-2;
    scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
    scale=sqrt((scale+1)*ssr/df);
    logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
    Mscore2(i,2)=mean(logtpdf);
    Mscore3(i,2)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    
    
    %% post lasso with the asymptotic x-dependent selection of BCH (alg 2)
    if sum(ModelsToRun==3)>0
%         M=5000;
%         aux=NaN(M,1);
%         for rep=1:M
%             aux(rep)=Tin*max(abs(mean(x.*repmat(normrnd(0,1,Tin,1),1,k))));
%         end
%         % quantile=prctile(aux,100*(1-.1/log(max([T,k])))); % recommendation in BCCH Econometrica
%         quantile=prctile(aux,95); % recommendation in BCH World congress
%         rit=IteratedLassoBCH(y,x,ut,quantile,'algorithm2');
%         is=find(rit.blassoBCHxdep~=0);
        
        quantile=sqrt(Tin)*norminv(1 -.05/(2*k));
        rit=IteratedLassoBCH(y,x,ut,quantile,'algorithm2');
        is=find(rit.blassoBCHindep~=0);
        
        ZSTAR(i,is(l+1:end),3)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,3)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,3)=mean(logtpdf);
        Mscore3(i,3)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
    
    %% Square-root post lasso with the asymptotic x-dependent selection of BCH
    if sum(ModelsToRun==4)>0     
        ALPHA = 0.05;
        c = 1.1; 
        M=5000;
        aux=NaN(M,1);
        for rep=1:M
            gs = normrnd(0,1,Tin,1);
            aux(rep)=Tin*max(abs(mean(x.*repmat(gs,1,k))))/sqrt(mean(gs.^2));
        end
        quantile=prctile(aux,100*(1-ALPHA)); % recommendation in BCH World congress
         
        lambda = c*quantile;
        % lambda = c*sqrt(Tin)*norminv(1 - ( ALPHA/(2*k) ), 0,1);
        [betaSQ, sSQ] = SqrtLassoIterative([ut*1000 x], y, lambda, ones(k+l,1));
       
        is=find(betaSQ~=0);
        %is=find(rit.blassoBCHindep~=0);
        ZSTAR(i,is(l+1:end),4)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,4)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,4)=mean(logtpdf);
        Mscore3(i,4)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
    
    %% post lasso with 2-fold CV
    if sum(ModelsToRun==5)>0
        Kfold=2;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=PostLassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),5)=1;
        
        % squared deviations
        if strcmp(dataset,'EminentDomain')
            r=ols1pinv(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        else
            r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        end
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,5)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        if strcmp(dataset,'EminentDomain')
            scale=diag([ones(Tout,1) UX(SmplOut,is)]*pinv(r.XX)*[ones(Tout,1) UX(SmplOut,is)]');
        else
            scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        end
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,5)=mean(logtpdf);
        Mscore3(i,5)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
    
    %% post lasso with 5-fold CV
    if sum(ModelsToRun==6)>0
        Kfold=5;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=PostLassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),6)=1;
        
        % squared deviations
        r=ols1pinv(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,6)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,6)=mean(logtpdf);
        Mscore3(i,6)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
    
    %% post lasso with 10-fold CV
    if sum(ModelsToRun==7)>0
        Kfold=10;
        if exist('pU'); pu=pU(SmplIn,:); else; pu=[]; end
        rCV=PostLassoCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplInU,:),Kfold,maxS,pu);
        is=find(bfix(1+rCV.s+l,:)~=0);
        ZSTAR(i,is(l+1:end),7)=1;
        
        % squared deviations
        r=ols1pinv(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,7)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,7)=mean(logtpdf);
        Mscore3(i,7)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
    
    %% PC regression with nf factors
    if sum(ModelsToRun==8)>0 
        nf=2;
        [v,d]=eigs(x'*x/Tin,nf);
        fin=x*v*diag(1./sqrt(diag(d)));
        fout=X(SmplOut,:)*v;
        fout=(X(SmplOut,:)-Mx)./Sx*v*diag(1./sqrt(diag(d)));
         
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) fin]);
        Mfct=[ones(Tout,1) fout]*r.bhatols;
        SQdev2(i,8)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) fout]/r.XX*[ones(Tout,1) fout]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,8)=mean(logtpdf);
        Mscore3(i,8)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols))))); 
    end
    
end
timeCVlasso=toc

clearvars -except SQdev0 SQdev1 SQdev2 SQdev3 Mscore0 Mscore1 Mscore2 Mscore3 NEWMAXS

% save ../LassoResults/NEWforecasts/PostLassoFcstGaussFinance1.mat;