%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code used to produce the Log Predictive Score and the Mean Squared 
% Forecast Error in the SBR out-of-sample exercise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
rng(10)

load ../../../data/Micro2/Data1stStageEminentDomain_u80.mat
load ../../../data/Micro2/CvSamplesMicro2.mat
dataset='EminentDomain';
data=Data1stStageEminentDomain_u80; clear Data1stStageEminentDomain_u80;
pU=data(:,1:80);
data(:,1:80)=[];
l=0;
lns=0;
maxS=60;
Mcv = 5*20;


if l>0; origU=data(:,1:l); else; origU=[]; end
origX=data(:,l+1:end-1);
[T k]=size(origX);
origUX=[origU origX];
origY=data(:,end);
clear data

Nmodels=7;
ModelNames=['SBR5','   SBR10','   SBRCV2','   SBRCV5','   SBRCV10','   SS5','   SS10'];
ModelsToRun=[1:5];

Mscore2=zeros(Mcv,Nmodels); SQdev2=zeros(Mcv,Nmodels);
Mscore3=zeros(Mcv,Nmodels);
ZSTAR=zeros(Mcv,k,Nmodels);

tic
for i=1:Mcv
    i
    %% selection of the CV sample
    SmplIn  = EstSmplCv{i};
    SmplOut = EvalSmplCv{i};
    Tin=length(SmplIn);
    Tout=length(SmplOut);
    
    if strcmp(dataset,'abortion')|strcmp(dataset,'EminentDomain')
        pr=ols1(origY(SmplIn),[pU(SmplIn,:)]);
        Y=origY-pU*pr.bhatols;
        prx=olsblock(origX(SmplIn,:),[pU(SmplIn,:)]);
        X=origX-pU*prx.bhatols;
        if l>0
            pr=ols1(origU(SmplIn,:),[pU(SmplIn,:)]);
            U=origU-pU*pr.bhatols;
        else
            U=[];
        end
        UX=[U X];
    else
        Y=origY; X=origX; U=origU; UX=[U X];
    end
    
    %% standardization of the variables
    if l>0; [ut,Mu,Su] = zscore(U(SmplIn,lns+1:end),1); else; ut=[]; Mu=[]; Su=[]; end
    if lns>0; ut=[U(SmplIn,1:lns) ut]; Mu=[zeros(1,lns) Mu]; Su=[ones(1,lns) Su]; end
    [x,Mx,Sx] = zscore(X(SmplIn,:),1);
    [y,My,Sy] = zscore(Y(SmplIn,:),1);
    ux=[ut x];
    Mux=[Mu Mx];
    Sux=[Su Sx];
    
    
    %% SBR with 5 predictors
    if sum(ModelsToRun==1)>0        
        if l>0; uy=ut'*y; ux=ut'*x; else; uy=[]; ux=[]; end
        r=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,5);
        is=[1:l;find(r.Z)+l];
        ZSTAR(i,is(l+1:end),1)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        % SQdev2(i,1)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,1)=mean(logtpdf);
        Mscore3(i,1)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));  
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,1)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
        
    end
    
    
    %% SBR with 10 predictors
    if sum(ModelsToRun==2)>0
        if l>0; uy=ut'*y; ux=ut'*x; else; uy=[]; ux=[]; end
        r=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,10);
        is=[1:l;find(r.Z)+l];
        ZSTAR(i,is(l+1:end),2)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        % SQdev2(i,2)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,2)=mean(logtpdf);
        Mscore3(i,2)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));  

        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,2)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);        
        
    end
    
    
    %% SBR with 2-fold CV
    if sum(ModelsToRun==3)>0
        Kfold=2;
        if l==0; uy=[]; ux=[]; else; uy=ut'*y; ux=ut'*x; end
        rA=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,maxS);
        rB=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,0);
        
        rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),[],rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         if exist('pU'); 
%             rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplIn,:),rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         else
%             rCV=SBRCVK(y,x,ut,rA.lambda,rB.lambda,Kfold);
%         end
        rCV=SBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,rCV.lambda);
        
        is=[1:l;find(rCV.Z)+l];
        ZSTAR(i,is(l+1:end),3)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        % SQdev2(i,3)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,3)=mean(logtpdf);
        Mscore3(i,3)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));  
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,3)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
    end
    
    
    %% SBR with 5-fold CV
    if sum(ModelsToRun==4)>0
        Kfold=5;
        if l==0; uy=[]; ux=[]; else; uy=ut'*y; ux=ut'*x; end
        % rA=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,maxS); % no need to repeat this
        % rB=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,0);
        
        rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),[],rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         if exist('pU'); 
%             rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplIn,:),rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         else
%             rCV=SBRCVK(y,x,ut,rA.lambda,rB.lambda,Kfold);
%         end
        rCV=SBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,rCV.lambda);
        
        is=[1:l;find(rCV.Z)+l];
        ZSTAR(i,is(l+1:end),4)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        %SQdev2(i,4)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,4)=mean(logtpdf);
        Mscore3(i,4)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));   
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,4)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
    end
    
    
    %% SBR with 10-fold CV
    if sum(ModelsToRun==5)>0
        Kfold=10;
        if l==0; uy=[]; ux=[]; else; uy=ut'*y; ux=ut'*x; end
        % rA=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,maxS); 
        % rB=bisectionSBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,1,10,0);
        
        rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),[],rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         if exist('pU'); 
%             rCV=SBRCVK(origY(SmplIn),origX(SmplIn,:),origU(SmplIn,:),rA.lambda,rB.lambda,Kfold,pU(SmplIn,:));
%         else
%             rCV=SBRCVK(y,x,ut,rA.lambda,rB.lambda,Kfold);
%         end
        rCV=SBR(y'*y,x'*x,x'*y,ut'*ut,uy,ux,k,rCV.lambda);
        
        is=[1:l;find(rCV.Z)+l];
        ZSTAR(i,is(l+1:end),5)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        % SQdev2(i,5)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,5)=mean(logtpdf);
        Mscore3(i,5)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));   
        
        uniqueY=unique(origY);
        temp=zeros(Tout,length(uniqueY));
        for ii=1:length(uniqueY)
            temp(:,ii)=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((uniqueY(ii)-pU(SmplOut,:)*pr.bhatols-Mfct)./scale).^2/df);
        end
        [junk maxscore]=max(temp');
        SQdev2(i,5)=mean((Y(SmplOut)-uniqueY(maxscore)+pU(SmplOut,:)*pr.bhatols).^2);
    end
    
    
    %% Stationary solution with 5 predictors
    if sum(ModelsToRun==6)>0    
        
        [bfix]=Lars([ut*1000 x],y);
        r=StationarySolutionWithU(y,[ut x],k+l,5,max(eigs([ut x]'*[ut x]))+10,bfix(1+5+l,:)',y'*y,y'*[ut x],[ut x]'*[ut x],l);
    
        find(bfix(1+5+l,:)~=0)-find(r.b)'
        %r=StationarySolutionWithU(y,[ut x],k+l,5,max(eigs([ut x]'*[ut x]))+10,zeros(k+l,1),y'*y,y'*[ut x],[ut x]'*[ut x],l);
        is=[find(r.b)];      
        ZSTAR(i,is(l+1:end)-l,6)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,6)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,6)=mean(logtpdf);
        Mscore3(i,6)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));   
    end
    
    
    %% Stationary solution with 10 predictors
    if sum(ModelsToRun==7)>0        
        r=StationarySolutionWithU(y,[ut x],k+l,10,max(eigs([ut x]'*[ut x]))+10,zeros(k+l,1),y'*y,y'*[ut x],[ut x]'*[ut x],l);
        is=[find(r.b)];       
        ZSTAR(i,is(l+1:end)-l,7)=1;
        
        % squared deviations
        r=ols1(Y(SmplIn),[ones(Tin,1) UX(SmplIn,is)]);
        Mfct=[ones(Tout,1) UX(SmplOut,is)]*r.bhatols;
        SQdev2(i,7)=mean((Y(SmplOut)-Mfct).^2);
        
        % log predictive scores
        ssr=r.resols'*r.resols;
        df=Tin-length(r.bhatols)-2;
        scale=diag([ones(Tout,1) UX(SmplOut,is)]/r.XX*[ones(Tout,1) UX(SmplOut,is)]');
        scale=sqrt((scale+1)*ssr/df);
        logtpdf=gammaln(df/2+1/2)-gammaln(df/2)-.5*log(pi*df)-log(scale)-.5*(df+1)*log(1+((Y(SmplOut)-Mfct)./scale).^2/df);
        Mscore2(i,7)=mean(logtpdf);
        Mscore3(i,7)=mean(logofnormpdf(Y(SmplOut),Mfct,sqrt(ssr/(Tin-length(r.bhatols)))));   
    end
    
end
timeCVL0=toc

clearvars -except SQdev0 SQdev1 SQdev2 SQdev3 Mscore0 Mscore1 Mscore2 Mscore3 NEWMAXS ModelNames

% save ../SBRResults/NEWforecasts/SBRFcstMicro2.mat;