function [S]=TBFMS_fast(y,x,u,alpha,c_tau,type);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Forward selection as described in "Testing-Based Forward Model Selection,"
% by Kozbur, published in Econometrica
%
% y: response variable
% x: regressors
% u: regressors always included
% alpha and c_tau: tuning parameters
% type: 1, 2, 3 or 4, as in the original Econometrica paper
%
% This code is based on Kozbur's original code for his AER P&P paper
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% dimensions 
[n,p]=size(x);
l=size(u,2);    

% preliminary calculations
xu=[x u]; xuxu=xu'*xu; xuy=xu'*y; yy=y'*y;

% initialization
S=[];
S_candidates = [1:p];
Z=1;

% forward model selection
while Z==1
    T=zeros(p,1);
    W=zeros(p,1);
    L=Inf(p,1);
    for j=S_candidates
        indxjS = [j S p+1:p+l];
        indxS = [S p+1:p+l];
        b=xuxu(indxjS,indxjS)\xuy(indxjS); ssr=yy-xuy(indxjS)'*b;
        
        if type == 1 || type == 2 || type == 3
            e = y-xu(:,indxjS)*b;
            [se,~,PSI] = hetero_se(xu(:,indxjS),e,inv(xuxu(indxjS,indxjS)));
            W(j) = abs(b(1))/se(1);
            eta = [1; -xuxu(indxS,indxS)\xuxu(indxS,j)];
            tau = abs(eta')*sqrt(diag(PSI))/sqrt(eta'*PSI*eta);
            L(j)=ssr;
        elseif type == 4
            se = sqrt(ssr/n)*sqrt(diag(inv(xuxu(indxjS,indxjS))));
            W(j) = abs(b(1))/se(1);
        end
        
        % Four options
        if type == 1         % TBFMS-I
            T(j) = W(j) > c_tau*tau*norminv(1-alpha/(p+l));
        elseif type == 2     % TBFMS-II
            T(j) = W(j) > norminv(1-alpha/(p+l));
        elseif type == 3     % TBFMS-III
            T(j) = W(j) > norminv(1-alpha/(p+l));
        elseif type == 4     % TBFMS-IV
            T(j) = W(j) > norminv(1-alpha/(p+l));
        end
    end
    
    Selected_set = find(T == 1); %#ok
    if isempty(Selected_set)
        Z=0;
    else
        if type == 3
            jhat = find(L == min(L));
        else
            jhat = find(W == max(W));
        end
        S = [S,jhat]; %#ok
        if length(S) >n-3, Z=0; end
        S_candidates = setdiff(S_candidates,jhat);
    end
end

% regression coefficients and fitted values of selected model
% if isempty(S)
%     if isempty(u)
%         theta = []; yhat = zeros(n,1);
%     else
%         theta = u\y; yhat = u*theta;        % modified to account for baseline covariates
%     end
% else
%     theta = [x(:,S) u]\y;
%     yhat = [x(:,S) u]*theta;
% end


% HC1 standard error
function [se,V,PSI] = hetero_se(x,e,invxx)
[n,k] = size(x);
PSI= (x.*((e.^2)*ones(1,k)))'*x/n;
V=[invxx*PSI*invxx]*n;
se=sqrt(diag(V));